<?php

namespace Lc\CaracoleBundle\Builder\Order;

use Doctrine\ORM\EntityManagerInterface;
use Lc\CaracoleBundle\Model\Order\OrderProductInterface;
use Lc\CaracoleBundle\Repository\Order\OrderProductStore;
use Lc\CaracoleBundle\Resolver\Price\PriceResolver;
use Lc\CaracoleBundle\Solver\Order\OrderProductSolver;
use Lc\CaracoleBundle\Solver\Price\PriceSolver;
use Lc\CaracoleBundle\Solver\Product\ProductFamilySolver;
use Lc\CaracoleBundle\Solver\Product\ProductSolver;

class OrderProductBuilder
{
    protected EntityManagerInterface $entityManager;
    protected PriceSolver $priceSolver;
    protected OrderProductStore $orderProductStore;
    protected ProductSolver $productSolver;
    protected ProductFamilySolver $productFamilySolver;
    protected OrderProductSolver $orderProductSolver;

    public function __construct(
            EntityManagerInterface $entityManager,
            PriceSolver $priceSolver,
            OrderProductStore $orderProductStore,
            ProductSolver $productSolver,
            OrderProductSolver $orderProductSolver,
            ProductFamilySolver $productFamilySolver
    ) {
        $this->entityManager = $entityManager;
        $this->priceSolver = $priceSolver;
        $this->orderProductStore = $orderProductStore;
        $this->productSolver = $productSolver;
        $this->orderProductSolver = $orderProductSolver;
        $this->productFamilySolver = $productFamilySolver;
    }

    public function init(OrderProductInterface $orderProduct) :OrderProductInterface
    {
        $orderProduct->setTitle($this->orderProductSolver->getTitleOrderShop($orderProduct));
        $orderProduct->setPrice($this->priceSolver->getPrice($orderProduct->getProduct()));
        $orderProduct->setBuyingPrice($this->priceSolver->getBuyingPrice($orderProduct->getProduct()));
        $orderProduct->setUnit($this->productSolver->getUnitInherited($orderProduct->getProduct()));
        $orderProduct->setTaxRate($this->productFamilySolver->getTaxRateInherited($orderProduct->getProduct()));
        $orderProduct->setQuantityProduct($this->productSolver->getQuantityInherited($orderProduct->getProduct()));

        return $orderProduct;
    }
}
