<?php

namespace Lc\CaracoleBundle\Field\Filter\Order;

use EasyCorp\Bundle\EasyAdminBundle\Dto\FieldDto;
use Lc\CaracoleBundle\Solver\Order\OrderPaymentSolver;
use Lc\CaracoleBundle\Solver\Order\OrderStatusSolver;
use Lc\SovBundle\Field\Filter\FilterTrait;
use Lc\SovBundle\Repository\RepositoryQueryInterface;
use Lc\SovBundle\Translation\TranslatorAdmin;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Contracts\Translation\TranslatorTrait;

/**
 * @author La clic ! <contact@laclic.fr>
 */
class OrderShopOrderStatusFilter
{
    use FilterTrait;

    protected $translatorAdmin;

    public function __construct(TranslatorAdmin $translatorAdmin)
    {
        $this->translatorAdmin = $translatorAdmin;
    }

    public function buildProperty(FormBuilderInterface $builder, FieldDto $fieldDto, $options = array())
    {
        $builder->add(
            $fieldDto->getProperty(),
            ChoiceType::class,
            [
                'placeholder' => '--',
                'choices' =>
                    $this->translatorAdmin->transChoices(
                        OrderStatusSolver::getOrderStatusChoices(),
                        'OrderStatus',
                        'status'
                    ),
                'required' => false,
                'attr' => array(
                    'class' => 'select2 input-sm',
                    'form' => 'filters-form',
                ),
            ]
        );
    }

    public function applyFilter(RepositoryQueryInterface $repositoryQuery, FieldDto $fieldDto, $filteredValue = null)
    {
        if ($filteredValue !== null) {
            $repositoryQuery->filterByStatus([$filteredValue]);
        }
    }

}
