<?php

namespace Lc\CaracoleBundle\Model\Order;

use Doctrine\Common\Collections\Collection;
use Lc\CaracoleBundle\Model\Address\AddressInterface;
use Lc\CaracoleBundle\Model\Config\TaxRateInterface;
use Lc\CaracoleBundle\Model\File\DocumentInterface;
use Lc\CaracoleBundle\Model\PointSale\PointSaleInterface;
use Lc\CaracoleBundle\Model\Section\SectionInterface;
use Lc\CaracoleBundle\Model\User\VisitorInterface;
use Lc\SovBundle\Model\Ticket\TicketInterface;
use Lc\SovBundle\Model\User\UserInterface;


interface OrderShopInterface
{
    public function getValidationDate(): ?\DateTimeInterface;

    public function setValidationDate(\DateTimeInterface $validationDate
    ): OrderShopModel;

    public function getUser(): ?UserInterface;

    public function setUser(?UserInterface $user): OrderShopModel;

    public function getInvoiceAddress(): ?AddressInterface;

    public function setInvoiceAddress(?AddressInterface $invoiceAddress): OrderShopModel;

    public function getInvoiceAddressText(): ?string;

    public function setInvoiceAddressText(string $invoiceAddressText): OrderShopModel;

    public function getComment(): ?string;

    public function setComment(?string $comment): OrderShopModel;

    public function getMeanPayment(): ?string;

    public function setMeanPayment(string $meanPayment): OrderShopModel;

    /**
     * @return Collection|OrderStatusHistoryInterface[]
     */
    public function getOrderStatusHistories(): Collection;

    public function addOrderStatusHistory(OrderStatusHistoryInterface $orderStatusHistory
    ): OrderShopModel;

    public function removeOrderStatusHistory(OrderStatusHistoryInterface $orderStatusHistory
    ): OrderShopModel;

    /**
     * @return Collection|OrderPaymentInterface[]
     */
    public function getOrderPayments($meanPayment = null): Collection;

    public function addOrderPayment(OrderPaymentInterface $orderPayment): OrderShopModel;

    public function removeOrderPayment(OrderPaymentInterface $orderPayment
    ): OrderShopModel;

    /**
     * @return Collection|OrderProductInterface[]
     */
    public function getOrderProducts(): Collection;

    public function addOrderProduct(OrderProductInterface $orderProduct): OrderShopModel;

    public function removeOrderProduct(OrderProductInterface $orderProduct
    ): OrderShopModel;

    public function getVisitor(): ?VisitorInterface;

    public function setVisitor(?VisitorInterface $visitor): OrderShopModel;

    public function getDeliveryInfos(): ?string;

    public function setDeliveryInfos(?string $deliveryInfos): OrderShopModel;

    public function getOrderStatus(): ?OrderStatusInterface;

    public function setOrderStatusProtected(?OrderStatusInterface $orderStatus
    ): OrderShopModel;

    /**
     * @return Collection|OrderReductionCartInterface[]
     */
    public function getOrderReductionCarts(): Collection;

    public function addOrderReductionCart(OrderReductionCartInterface $orderReductionCart
    ): OrderShopModel;

    public function removeOrderReductionCart(OrderReductionCartInterface $orderReductionCart
    ): OrderShopModel;

    /**
     * @return Collection|OrderReductionCreditInterface[]
     */
    public function getOrderReductionCredits(): Collection;

    public function addOrderReductionCredit(OrderReductionCreditInterface $orderReductionCredit
    ): OrderShopModel;

    public function removeOrderReductionCredit(OrderReductionCreditInterface $orderReductionCredit
    ): OrderShopModel;

    /**
     * @return Collection|DocumentInterface[]
     */
    public function getDocuments(): Collection;

    public function addDocument(DocumentInterface $document): OrderShopModel;

    public function removeDocument(DocumentInterface $document): OrderShopModel;

    /**
     * @return Collection|TicketInterface[]
     */
    public function getTickets(): Collection;

    public function addTicket(TicketInterface $ticket): OrderShopModel;

    public function removeTicket(TicketInterface $ticket): OrderShopModel;

    public function getSection(): ?SectionInterface;

    public function setSection(?SectionInterface $section): OrderShopModel;

    public function getCycleId(): ?int;

    public function setCycleId(?int $cycleId): OrderShopModel;

    public function getOrderShopCreatedAt(): ?\DateTimeInterface;

    public function setOrderShopCreatedAt(?\DateTimeInterface $orderShopCreatedAt
    ): OrderShopModel;

    public function getIdValidOrder(): ?int;

    public function setIdValidOrder(?int $idValidOrder): OrderShopModel;

    public function getDeliveryAddress(): ?AddressInterface;

    public function setDeliveryAddress(?AddressInterface $deliveryAddress
    ): OrderShopModel;

    public function getDeliveryAddressText(): ?string;

    public function setDeliveryAddressText(string $deliveryAddressText): OrderShopModel;

    public function getDeliveryPointSale(): ?PointSaleInterface;

    public function setDeliveryPointSale(?PointSaleInterface $deliveryPointSale
    ): OrderShopModel;

    public function getDeliveryType(): ?string;

    public function setDeliveryType(?string $deliveryType): OrderShopModel;

    public function getDeliveryPrice(): ?float;

    public function setDeliveryPrice(?float $deliveryPrice): OrderShopModel;

    public function getDeliveryTaxRate(): ?TaxRateInterface;

    public function setDeliveryTaxRate(?TaxRateInterface $deliveryTaxRate
    ): OrderShopModel;

    public function getReference(): ?string;

    public function setReference(?string $reference): OrderShopModel;

    public function getMainOrderShop(): ?self;

    public function setMainOrderShop(?OrderShopModel $mainOrderShop): OrderShopModel;

    /**
     * @return Collection|OrderShopInterface[]
     */
    public function getComplementaryOrderShops(): Collection;

    public function addComplementaryOrderShop(OrderShopModel $complementaryOrderShop
    ): OrderShopModel;

    public function removeComplementaryOrderShop(OrderShopModel $complementaryOrderShop
    ): OrderShopModel;

    public function getDeclineComplementaryOrderShop(): ?bool;

    public function setDeclineComplementaryOrderShop(?bool $declineComplementaryOrderShop
    ): OrderShopModel;

    public function getOrderAllowByAdmin(): ?bool;

    public function setOrderAllowByAdmin(?bool $orderAllowByAdmin): OrderShopModel;

    public function getHasReach(): ?int;

    public function setHasReach(?int $hasReach): OrderShopModel;

    public function getStatTotal(): ?float;

    public function setStatTotal(?float $statTotal): OrderShopModel;

    public function getStatTotalWithTax(): ?float;

    public function setStatTotalWithTax(?float $statTotalWithTax): OrderShopModel;

    public function getStatTotalOrderProductsWithReductions(): ?float;

    public function setStatTotalOrderProductsWithReductions(?float $statTotalOrderProductsWithReductions
    ): OrderShopModel;

    public function getStatTotalOrderProductsWithTaxAndReductions(): ?float;

    public function setStatTotalOrderProductsWithTaxAndReductions(?float $statTotalOrderProductsWithTaxAndReductions
    ): OrderShopModel;

    public function getStatMarginOrderProductsWithReductions(): ?float;

    public function setStatMarginOrderProductsWithReductions(?float $statMarginOrderProductsWithReductions
    ): OrderShopModel;

    public function getStatDeliveryPriceWithReduction(): ?float;

    public function setStatDeliveryPriceWithReduction(?float $statDeliveryPriceWithReduction
    ): OrderShopModel;

    public function getStatDeliveryPriceWithTaxAndReduction(): ?float;

    public function setStatDeliveryPriceWithTaxAndReduction(?float $statDeliveryPriceWithTaxAndReduction
    ): OrderShopModel;
}
