<?php

namespace Lc\CaracoleBundle\Model\Product;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Lc\CaracoleBundle\Doctrine\Extension\FilterSectionInterface;
use Lc\CaracoleBundle\Doctrine\Extension\PriceInterface;
use Lc\CaracoleBundle\Doctrine\Extension\ProductPropertyTrait;
use Lc\CaracoleBundle\Doctrine\Extension\ProductPropertyInterface;
use Lc\CaracoleBundle\Model\Reduction\ReductionCatalogInterface;
use Lc\SovBundle\Doctrine\Pattern\AbstractFullEntity;
use Lc\SovBundle\Model\File\FileInterface;


/**
 * @ORM\MappedSuperclass()
 */
abstract class ProductFamilyModel extends AbstractFullEntity implements ProductPropertyInterface, PriceInterface, ProductFamilyInterface, FilterSectionInterface
{
    use ProductPropertyTrait;

    const BEHAVIOR_COUNT_STOCK_BY_MEASURE = 'by-measure';
    const BEHAVIOR_COUNT_STOCK_BY_PRODUCT_FAMILY = 'by-product-family';
    const BEHAVIOR_COUNT_STOCK_BY_PRODUCT = 'by-product';

    const BEHAVIOR_DISPLAY_SALE_BY_MEASURE = 'by-measure';
    const BEHAVIOR_DISPLAY_SALE_BY_QUANTITY = 'by-quantity';

    const WARNING_MESSAGE_TYPE_SUCCESS = 'success';
    const WARNING_MESSAGE_TYPE_ERROR = 'error';
    const WARNING_MESSAGE_TYPE_WARNING = 'warning';
    const WARNING_MESSAGE_TYPE_INFO = 'info';

    const BEHAVIOR_ADD_TO_CART_SIMPLE = 'simple';
    const BEHAVIOR_ADD_TO_CART_MULTIPLE = 'multiple';

    const BEHAVIOR_PRICE_BY_PIECE = 'by-piece';
    const BEHAVIOR_PRICE_BY_REFERENCE_UNIT = 'by-reference-unit';

    const QUALITY_LABEL_AB = 'ab';
    const QUALITY_LABEL_NP = 'nature-progres';
    const QUALITY_LABEL_HVE = 'hve';
    const QUALITY_LABEL_TVVR = 'tvvr';
    const QUALITY_LABEL_AOC = 'aoc';
    const QUALITY_LABEL_AOP = 'aop';
    const QUALITY_LABEL_IGP = 'igp';

    static $organicLabels = [
            self::QUALITY_LABEL_AB,
            self::QUALITY_LABEL_NP,
            self::QUALITY_LABEL_HVE,
            self::QUALITY_LABEL_TVVR
    ];
    static $geographicLabels = [
            self::QUALITY_LABEL_AOC,
            self::QUALITY_LABEL_AOP,
            self::QUALITY_LABEL_IGP,
    ];

    const TYPE_EXPIRATION_DATE_DLC = 'dlc';
    const TYPE_EXPIRATION_DATE_DDM = 'ddm';
    const TYPE_EXPIRATION_DATE_DLUO = 'dluo';

    const BEHAVIOR_EXPIRATION_DATE_BY_PRODUCT_FAMILY = 'by-product-family';
    const BEHAVIOR_EXPIRATION_DATE_BY_PRODUCT = 'by-product';

    // @TODO : Champ hydraté par ProductFamilyBuilder ?
    protected $reductionCatalog;

    /**
     * @ORM\ManyToMany(targetEntity="Lc\CaracoleBundle\Model\Product\ProductCategoryInterface", inversedBy="productFamilies")
     */
    protected $productCategories;

    /**
     * @ORM\Column(type="boolean")
     */
    protected $activeProducts;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    protected $productsType;

    /**
     * @ORM\Column(type="boolean", nullable=true)
     */
    protected $productsQuantityAsTitle;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    protected $quantityLabel;

    /**
     * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\Product\ProductInterface", mappedBy="productFamily", orphanRemoval=true, cascade={"persist"})
     * @ORM\OrderBy({"position" = "ASC"})
     */
    protected $products;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    protected $subtitle;

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    protected $warningMessage;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    protected $warningMessageType;

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    protected $note;

    /**
     * @ORM\Column(type="string", length=31, nullable=true)
     */
    protected $behaviorOutOfStock;

    /**
     * @ORM\Column(type="string", length=31)
     */
    protected $behaviorCountStock;

    /**
     * @ORM\Column(type="string", length=255)
     */
    protected $behaviorStockCycle;

    /**
     * @ORM\Column(type="string", length=255)
     */
    protected $behaviorDisplaySale;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    protected $exportTitle;

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    protected $exportNote;

    /**
     * @ORM\Column(type="date", nullable=true)
     */
    protected $propertyNoveltyExpirationDate;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    protected $propertyOrganicLabel;

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    protected $propertyAllergens;

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    protected $propertyComposition;

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    protected $propertyFragrances;

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    protected $propertyPackaging;

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    protected $propertyCharacteristics;

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    protected $propertyWeight;

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    protected $propertyQuantity;

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    protected $propertyFeature;


    /**
     * @ORM\Column(type="text", nullable=true)
     */
    protected $propertyVariety;

    /**
     * @ORM\Column(type="text", nullable=true)
     */
    protected $propertyAlcoholLevel;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    protected $behaviorExpirationDate;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    protected $typeExpirationDate;

    /**
     * @ORM\Column(type="string", length=32, nullable=true)
     */
    protected $behaviorAddToCart;

    /**
     * @ORM\Column(type="string", length=31)
     */
    protected $behaviorPrice;

    /**
     * @ORM\Column(type="boolean")
     */
    protected $saleStatus;

    /**
     * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\Product\ProductFamilySectionPropertyInterface", mappedBy="productFamily", cascade={"persist"})
     */
    protected $productFamilySectionProperties;

    /**
     * @ORM\ManyToOne(targetEntity="Lc\SovBundle\Model\File\FileInterface", cascade={"persist", "remove"})
     */
    protected $image;

    /**
     * @ORM\ManyToMany(targetEntity="Lc\CaracoleBundle\Model\Product\QualityLabelInterface", fetch="EAGER")
     */
    protected $qualityLabels;

    public function __construct()
    {
        $this->productCategories = new ArrayCollection();
        $this->products = new ArrayCollection();
        $this->qualityLabels = new ArrayCollection();
    }

    public function __toString()
    {
        return $this->getTitle();
    }

    public function getActiveProducts(): ?bool
    {
        return $this->activeProducts;
    }

    public function setActiveProducts(bool $activeProducts): self
    {
        $this->activeProducts = $activeProducts;

        return $this;
    }

    public function getProductsQuantityAsTitle(): ?bool
    {
        return $this->productsQuantityAsTitle;
    }

    public function setProductsQuantityAsTitle(bool $productsQuantityAsTitle): self
    {
        $this->productsQuantityAsTitle = $productsQuantityAsTitle;

        return $this;
    }

    public function getProductsType(): ?string
    {
        return $this->productsType;
    }

    public function setProductsType(?string $productsType): self
    {
        $this->productsType = $productsType;

        return $this;
    }

    public function getQuantityLabel(): ?string
    {
        return $this->quantityLabel;
    }

    public function setQuantityLabel(?string $quantityLabel): self
    {
        $this->quantityLabel = $quantityLabel;

        return $this;
    }

    /**
     * @return Collection|ProductInterface[]
     */
    public function getProducts(): Collection
    {
        return $this->products;
    }

    public function addProduct(ProductInterface $product): self
    {
        if (!$this->products->contains($product)) {
            $this->products[] = $product;
            $product->setProductFamily($this);
        }

        return $this;
    }

    public function removeProduct(ProductInterface $product): self
    {
        if ($this->products->contains($product)) {
            $this->products->removeElement($product);
            // set the owning side to null (unless already changed)
            if ($product->getProductFamily() === $this) {
                $product->setProductFamily(null);
            }
        }

        return $this;
    }

    public function getReductionCatalog(): ?ReductionCatalogInterface
    {
        return $this->reductionCatalog;
    }

    public function setReductionCatalog(?ReductionCatalogInterface $reductionCatalog): self
    {
        $this->reductionCatalog = $reductionCatalog;

        return $this;
    }

    /**
     * @return Collection|ProductCategoryInterface[]
     */
    public function getProductCategories(): Collection
    {
        return $this->productCategories;
    }

    public function initProductCategories()
    {
        $this->productCategories = new ArrayCollection();
    }

    public function addProductCategory(ProductCategoryInterface $productCategory): self
    {
        if (!$this->productCategories->contains($productCategory)) {
            $this->productCategories[] = $productCategory;
        }

        return $this;
    }

    public function removeProductCategory(ProductCategoryInterface $productCategory): self
    {
        if ($this->productCategories->contains($productCategory)) {
            $this->productCategories->removeElement($productCategory);
        }

        return $this;
    }

    public function getSubtitle(): ?string
    {
        return $this->subtitle;
    }

    public function setSubtitle(?string $subtitle): self
    {
        $this->subtitle = $subtitle;

        return $this;
    }

    public function getWarningMessage(): ?string
    {
        return $this->warningMessage;
    }

    public function setWarningMessage(?string $warningMessage): self
    {
        $this->warningMessage = $warningMessage;

        return $this;
    }

    public function getWarningMessageType(): ?string
    {
        return $this->warningMessageType;
    }

    public function setWarningMessageType(?string $warningMessageType): self
    {
        $this->warningMessageType = $warningMessageType;

        return $this;
    }

    public function getNote(): ?string
    {
        return $this->note;
    }

    public function setNote(?string $note): self
    {
        $this->note = $note;

        return $this;
    }

    public function getBehaviorOutOfStock(): ?string
    {
        return $this->behaviorOutOfStock;
    }

    public function setBehaviorOutOfStock(?string $behaviorOutOfStock): self
    {
        $this->behaviorOutOfStock = $behaviorOutOfStock;

        return $this;
    }

    public function getBehaviorCountStock(): ?string
    {
        return $this->behaviorCountStock;
    }


    public function setBehaviorCountStock(string $behaviorCountStock): self
    {
        $this->behaviorCountStock = $behaviorCountStock;

        return $this;
    }

    public function getExportTitle(): ?string
    {
        return $this->exportTitle;
    }

    public function setExportTitle(?string $exportTitle): self
    {
        $this->exportTitle = $exportTitle;

        return $this;
    }

    public function getExportNote(): ?string
    {
        return $this->exportNote;
    }

    public function setExportNote(?string $exportNote): self
    {
        $this->exportNote = $exportNote;

        return $this;
    }

    public function getBehaviorStockCycle(): ?string
    {
        return $this->behaviorStockCycle;
    }

    public function setBehaviorStockCycle(string $behaviorStockCycle): self
    {
        $this->behaviorStockCycle = $behaviorStockCycle;

        return $this;
    }

    public function getBehaviorDisplaySale(): ?string
    {
        return $this->behaviorDisplaySale;
    }

    public function setBehaviorDisplaySale(string $behaviorDisplaySale): self
    {
        $this->behaviorDisplaySale = $behaviorDisplaySale;

        return $this;
    }

    public function getPropertyNoveltyExpirationDate(): ?\DateTimeInterface
    {
        return $this->propertyNoveltyExpirationDate;
    }

    public function setPropertyNoveltyExpirationDate(?\DateTimeInterface $propertyNoveltyExpirationDate): self
    {
        $this->propertyNoveltyExpirationDate = $propertyNoveltyExpirationDate;

        return $this;
    }

    public function getPropertyOrganicLabel(): ?string
    {
        return $this->propertyOrganicLabel;
    }

    public function setPropertyOrganicLabel(?string $propertyOrganicLabel): self
    {
        $this->propertyOrganicLabel = $propertyOrganicLabel;

        return $this;
    }

    public function getPropertyAllergens(): ?string
    {
        return $this->propertyAllergens;
    }

    public function setPropertyAllergens(?string $propertyAllergens): self
    {
        $this->propertyAllergens = $propertyAllergens;

        return $this;
    }

    public function getPropertyComposition(): ?string
    {
        return $this->propertyComposition;
    }

    public function setPropertyComposition(?string $propertyComposition): self
    {
        $this->propertyComposition = $propertyComposition;

        return $this;
    }

    public function getPropertyFragrances(): ?string
    {
        return $this->propertyFragrances;
    }

    public function setPropertyFragrances(?string $propertyFragrances): self
    {
        $this->propertyFragrances = $propertyFragrances;

        return $this;
    }

    public function getBehaviorExpirationDate(): ?string
    {
        return $this->behaviorExpirationDate;
    }

    public function setBehaviorExpirationDate(?string $behaviorExpirationDate): self
    {
        $this->behaviorExpirationDate = $behaviorExpirationDate;

        return $this;
    }

    public function getTypeExpirationDate(): ?string
    {
        return $this->typeExpirationDate;
    }

    public function setTypeExpirationDate(?string $typeExpirationDate): self
    {
        $this->typeExpirationDate = $typeExpirationDate;

        return $this;
    }

    public function getPropertyWeight(): ?string
    {
        return $this->propertyWeight;
    }

    public function setPropertyWeight(?string $propertyWeight): self
    {
        $this->propertyWeight = $propertyWeight;

        return $this;
    }

    public function getPropertyQuantity(): ?string
    {
        return $this->propertyQuantity;
    }

    public function setPropertyQuantity(?string $propertyQuantity): self
    {
        $this->propertyQuantity = $propertyQuantity;

        return $this;
    }

    public function getPropertyVariety(): ?string
    {
        return $this->propertyVariety;
    }

    public function setPropertyVariety(?string $propertyVariety): self
    {
        $this->propertyVariety = $propertyVariety;

        return $this;
    }

    public function getPropertyFeature(): ?string
    {
        return $this->propertyFeature;
    }

    public function setPropertyFeature(?string $propertyFeature): self
    {
        $this->propertyFeature = $propertyFeature;

        return $this;
    }

    public function getPropertyAlcoholLevel(): ?string
    {
        return $this->propertyAlcoholLevel;
    }

    public function setPropertyAlcoholLevel(?string $propertyAlcoholLevel): self
    {
        $this->propertyAlcoholLevel = $propertyAlcoholLevel;

        return $this;
    }

    public function getPropertyPackaging(): ?string
    {
        return $this->propertyPackaging;
    }

    public function setPropertyPackaging(?string $propertyPackaging): self
    {
        $this->propertyPackaging = $propertyPackaging;

        return $this;
    }

    public function getPropertyCharacteristics(): ?string
    {
        return $this->propertyCharacteristics;
    }

    public function setPropertyCharacteristics(?string $propertyCharacteristics): self
    {
        $this->propertyCharacteristics = $propertyCharacteristics;

        return $this;
    }

    public function getBehaviorAddToCart(): ?string
    {
        return $this->behaviorAddToCart;
    }

    public function setBehaviorAddToCart(?string $behaviorAddToCart): self
    {
        $this->behaviorAddToCart = $behaviorAddToCart;

        return $this;
    }


    public function getBehaviorPrice(): ?string
    {
        return $this->behaviorPrice;
    }

    public function setBehaviorPrice(?string $behaviorPrice): self
    {
        $this->behaviorPrice = $behaviorPrice;

        return $this;
    }

    public function getSaleStatus(): ?bool
    {
        return $this->saleStatus;
    }

    public function setSaleStatus(bool $saleStatus): self
    {
        $this->saleStatus = $saleStatus;

        return $this;
    }

    public function getImage(): ?FileInterface
    {
        return $this->image;
    }

    public function setImage(?FileInterface $image): self
    {
        $this->image = $image;

        return $this;
    }

    /**
     * @return Collection|ProductFamilySectionPropertyInterface[]
     */
    public function getProductFamilySectionProperties(): Collection
    {
        return $this->productFamilySectionProperties;
    }

    public function addProductFamilySectionProperty(ProductFamilySectionPropertyInterface $productFamilySectionProperty): self
    {
        if (!$this->productFamilySectionProperties->contains($productFamilySectionProperty)) {
            $this->productFamilySectionProperties[] = $productFamilySectionProperty;
            $productFamilySectionProperty->setProductFamily($this);
        }

        return $this;
    }

    public function removeProductFamilySectionProperty(ProductFamilySectionPropertyInterface $productFamilySectionProperty): self
    {
        if ($this->productFamilySectionProperties->removeElement($productFamilySectionProperty)) {
            // set the owning side to null (unless already changed)
            if ($productFamilySectionProperty->getProductFamily() === $this) {
                $productFamilySectionProperty->setProductFamily(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection|QualityLabelInterface[]
     */
    public function getQualityLabels(): Collection
    {
        return $this->qualityLabels;
    }

    public function addQualityLabel(QualityLabelInterface $qualityLabel): self
    {
        if (!$this->qualityLabels->contains($qualityLabel)) {
            $this->qualityLabels[] = $qualityLabel;
        }

        return $this;
    }

    public function removeQualityLabel(QualityLabelInterface $qualityLabel): self
    {
        $this->qualityLabels->removeElement($qualityLabel);

        return $this;
    }


}
